<?php

namespace Drupal\wsolution\Vcr;

use VCR\Cassette;
use VCR\Request;
use VCR\Response;

/**
 * A Cassette records and plays back pairs of Requests and Responses in a
 * Storage.
 */
class WSCassette extends Cassette {

  protected $mode;

  protected $blacklist;

  public function setMode($mode) {
    $this->mode = $mode;
  }

  public function setBlacklist($blacklist) {
    $this->blacklist = $blacklist;
  }

  /**
   * Records a request and response pair.
   *
   * @param Request $request request to record
   * @param Response $response response to record
   */
  public function record(Request $request, Response $response, int $index = 0): void {

    if ($this->mode === 'all_request') {
      // Do not record if url is blacklisted.
      if ($this->filterBlacklist($request)) {
        return;
      }
    }

    if ($this->hasResponse($request)) {
      return;
    }

    $recording['request'] = $request->toArray();
    if (!empty($response)) {
      $recording['response'] = $response->toArray();
    }

    $this->storage->storeRecording($recording);
  }

  /**
   * @param \Drupal\Core\Config\ImmutableConfig $config
   * @param \VCR\Request $request
   *
   * @return bool|void
   */
  private function filterBlacklist(Request $request) {
    if (empty($this->blacklist)) {
      return FALSE;
    }
    $blacklist = explode(PHP_EOL, $this->blacklist);
    // @todo deport drupal service.
    $service_path = \Drupal::service('path.matcher');
    foreach ($blacklist as $black) {
      $black_url_port = explode(':', $black);
      // Test port.
      if (!empty($black_url_port[2])) {
        $curl_options = $request->getCurlOptions();
        // The path matching and the port.
        if ($curl_options[CURLOPT_PORT] == $black_url_port[2] && $service_path->matchPath($request->getUrl(), $black_url_port[0] . ':' . $black_url_port[1])) {
          return TRUE;
        }
        continue;
      }
      // The path is matching.
      if ($service_path->matchPath($request->getUrl(), $black)) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
